@@ -89,8 +89,7 @@ impl FieldElementImpl {
     }
 
     pub fn is_odd(&self) -> Choice {
-        debug_assert!(self.normalized);
-        self.value.is_odd()
+        self.normalize().value.is_odd()
     }
 
     pub fn negate(&self, magnitude: u32) -> Self {
